@@ -2,8 +2,8 @@
 *
 *  =========== DOCUMENTATION ===========
 *
-* Online html documentation available at 
-*            http://www.netlib.org/lapack/explore-html/ 
+* Online html documentation available at
+*            http://www.netlib.org/lapack/explore-html/
 *
 *> \htmlonly
 *> Download SORCSD2BY1 + dependencies
@@ -21,7 +21,7 @@
 *       SUBROUTINE SORCSD2BY1( JOBU1, JOBU2, JOBV1T, M, P, Q, X11, LDX11,
 *                              X21, LDX21, THETA, U1, LDU1, U2, LDU2, V1T,
 *                              LDV1T, WORK, LWORK, IWORK, INFO )
-* 
+*
 *       .. Scalar Arguments ..
 *       CHARACTER          JOBU1, JOBU2, JOBV1T
 *       INTEGER            INFO, LDU1, LDU2, LDV1T, LWORK, LDX11, LDX21,
@@ -33,8 +33,8 @@
 *      $                   X11(LDX11,*), X21(LDX21,*)
 *       INTEGER            IWORK(*)
 *       ..
-*    
-* 
+*
+*
 *> \par Purpose:
 *> =============
 *>
@@ -44,18 +44,19 @@
 *> orthonormal columns that has been partitioned into a 2-by-1 block
 *> structure:
 *>
-*>                                [  I  0  0 ]
+*>                                [  I1 0  0 ]
 *>                                [  0  C  0 ]
 *>          [ X11 ]   [ U1 |    ] [  0  0  0 ]
 *>      X = [-----] = [---------] [----------] V1**T .
 *>          [ X21 ]   [    | U2 ] [  0  0  0 ]
 *>                                [  0  S  0 ]
-*>                                [  0  0  I ]
-*> 
+*>                                [  0  0  I2]
+*>
 *> X11 is P-by-Q. The orthogonal matrices U1, U2, and V1 are P-by-P,
 *> (M-P)-by-(M-P), and Q-by-Q, respectively. C and S are R-by-R
 *> nonnegative diagonal matrices satisfying C^2 + S^2 = I, in which
-*> R = MIN(P,M-P,Q,M-Q).
+*> R = MIN(P,M-P,Q,M-Q). I1 is a K1-by-K1 identity matrix and I2 is a
+*> K2-by-K2 identity matrix, where K1 = MAX(Q+P-M,0), K2 = MAX(Q-P,0).
 *> \endverbatim
 *
 *  Arguments:
@@ -218,10 +219,10 @@
 *  Authors:
 *  ========
 *
-*> \author Univ. of Tennessee 
-*> \author Univ. of California Berkeley 
-*> \author Univ. of Colorado Denver 
-*> \author NAG Ltd. 
+*> \author Univ. of Tennessee
+*> \author Univ. of California Berkeley
+*> \author Univ. of Colorado Denver
+*> \author NAG Ltd.
 *
 *> \date July 2012
 *
@@ -232,7 +233,7 @@
      $                       X21, LDX21, THETA, U1, LDU1, U2, LDU2, V1T,
      $                       LDV1T, WORK, LWORK, IWORK, INFO )
 *
-*  -- LAPACK computational routine (version 3.6.0) --
+*  -- LAPACK computational routine (version 3.7.0) --
 *  -- LAPACK is a software package provided by Univ. of Tennessee,    --
 *  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
 *     July 2012
@@ -248,7 +249,7 @@
      $                   X11(LDX11,*), X21(LDX21,*)
       INTEGER            IWORK(*)
 *     ..
-*  
+*
 *  =====================================================================
 *
 *     .. Parameters ..
@@ -264,6 +265,9 @@
      $                   LWORKMIN, LWORKOPT, R
       LOGICAL            LQUERY, WANTU1, WANTU2, WANTV1T
 *     ..
+*     .. Local Arrays ..
+      REAL               DUM1(1), DUM2(1,1)
+*     ..
 *     .. External Subroutines ..
       EXTERNAL           SBBCSD, SCOPY, SLACPY, SLAPMR, SLAPMT, SORBDB1,
      $                   SORBDB2, SORBDB3, SORBDB4, SORGLQ, SORGQR,
@@ -296,11 +300,11 @@
          INFO = -8
       ELSE IF( LDX21 .LT. MAX( 1, M-P ) ) THEN
          INFO = -10
-      ELSE IF( WANTU1 .AND. LDU1 .LT. P ) THEN
+      ELSE IF( WANTU1 .AND. LDU1 .LT. MAX( 1, P ) ) THEN
          INFO = -13
-      ELSE IF( WANTU2 .AND. LDU2 .LT. M - P ) THEN
+      ELSE IF( WANTU2 .AND. LDU2 .LT. MAX( 1, M - P ) ) THEN
          INFO = -15
-      ELSE IF( WANTV1T .AND. LDV1T .LT. Q ) THEN
+      ELSE IF( WANTV1T .AND. LDV1T .LT. MAX( 1, Q ) ) THEN
          INFO = -17
       END IF
 *
@@ -342,98 +346,124 @@
          IORBDB = ITAUQ1 + MAX( 1, Q )
          IORGQR = ITAUQ1 + MAX( 1, Q )
          IORGLQ = ITAUQ1 + MAX( 1, Q )
+         LORGQRMIN = 1
+         LORGQROPT = 1
+         LORGLQMIN = 1
+         LORGLQOPT = 1
          IF( R .EQ. Q ) THEN
-            CALL SORBDB1( M, P, Q, X11, LDX11, X21, LDX21, THETA, 0, 0,
-     $                    0, 0, WORK, -1, CHILDINFO )
+            CALL SORBDB1( M, P, Q, X11, LDX11, X21, LDX21, THETA,
+     $                    DUM1, DUM1, DUM1, DUM1, WORK, -1,
+     $                    CHILDINFO )
             LORBDB = INT( WORK(1) )
-            IF( P .GE. M-P ) THEN
-               CALL SORGQR( P, P, Q, U1, LDU1, 0, WORK(1), -1,
+            IF( WANTU1 .AND. P .GT. 0 ) THEN
+               CALL SORGQR( P, P, Q, U1, LDU1, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, P )
-               LORGQROPT = INT( WORK(1) )
-            ELSE
-               CALL SORGQR( M-P, M-P, Q, U2, LDU2, 0, WORK(1), -1,
+               LORGQRMIN = MAX( LORGQRMIN, P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            ENDIF
+            IF( WANTU2 .AND. M-P .GT. 0 ) THEN
+               CALL SORGQR( M-P, M-P, Q, U2, LDU2, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, M-P )
-               LORGQROPT = INT( WORK(1) )
+               LORGQRMIN = MAX( LORGQRMIN, M-P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTV1T .AND. Q .GT. 0 ) THEN
+               CALL SORGLQ( Q-1, Q-1, Q-1, V1T, LDV1T,
+     $                      DUM1, WORK(1), -1, CHILDINFO )
+               LORGLQMIN = MAX( LORGLQMIN, Q-1 )
+               LORGLQOPT = MAX( LORGLQOPT, INT( WORK(1) ) )
             END IF
-            CALL SORGLQ( MAX(0,Q-1), MAX(0,Q-1), MAX(0,Q-1), V1T, LDV1T,
-     $                   0, WORK(1), -1, CHILDINFO )
-            LORGLQMIN = MAX( 1, Q-1 )
-            LORGLQOPT = INT( WORK(1) )
             CALL SBBCSD( JOBU1, JOBU2, JOBV1T, 'N', 'N', M, P, Q, THETA,
-     $                   0, U1, LDU1, U2, LDU2, V1T, LDV1T, 0, 1, 0, 0,
-     $                   0, 0, 0, 0, 0, 0, WORK(1), -1, CHILDINFO )
+     $                   DUM1, U1, LDU1, U2, LDU2, V1T, LDV1T, DUM2,
+     $                   1, DUM1, DUM1, DUM1, DUM1, DUM1,
+     $                   DUM1, DUM1, DUM1, WORK(1), -1, CHILDINFO
+     $                 )
             LBBCSD = INT( WORK(1) )
          ELSE IF( R .EQ. P ) THEN
-            CALL SORBDB2( M, P, Q, X11, LDX11, X21, LDX21, THETA, 0, 0,
-     $                    0, 0, WORK(1), -1, CHILDINFO )
+            CALL SORBDB2( M, P, Q, X11, LDX11, X21, LDX21, THETA,
+     $                    DUM1, DUM1, DUM1, DUM1, WORK(1), -1,
+     $                    CHILDINFO )
             LORBDB = INT( WORK(1) )
-            IF( P-1 .GE. M-P ) THEN
-               CALL SORGQR( P-1, P-1, P-1, U1(2,2), LDU1, 0, WORK(1),
-     $                      -1, CHILDINFO )
-               LORGQRMIN = MAX( 1, P-1 )
-               LORGQROPT = INT( WORK(1) )
-            ELSE
-               CALL SORGQR( M-P, M-P, Q, U2, LDU2, 0, WORK(1), -1,
+            IF( WANTU1 .AND. P .GT. 0 ) THEN
+               CALL SORGQR( P-1, P-1, P-1, U1(2,2), LDU1, DUM1,
+     $                      WORK(1), -1, CHILDINFO )
+               LORGQRMIN = MAX( LORGQRMIN, P-1 )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTU2 .AND. M-P .GT. 0 ) THEN
+               CALL SORGQR( M-P, M-P, Q, U2, LDU2, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, M-P )
-               LORGQROPT = INT( WORK(1) )
+               LORGQRMIN = MAX( LORGQRMIN, M-P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTV1T .AND. Q .GT. 0 ) THEN
+               CALL SORGLQ( Q, Q, R, V1T, LDV1T, DUM1, WORK(1), -1,
+     $                      CHILDINFO )
+               LORGLQMIN = MAX( LORGLQMIN, Q )
+               LORGLQOPT = MAX( LORGLQOPT, INT( WORK(1) ) )
             END IF
-            CALL SORGLQ( Q, Q, R, V1T, LDV1T, 0, WORK(1), -1,
-     $                   CHILDINFO )
-            LORGLQMIN = MAX( 1, Q )
-            LORGLQOPT = INT( WORK(1) )
             CALL SBBCSD( JOBV1T, 'N', JOBU1, JOBU2, 'T', M, Q, P, THETA,
-     $                   0, V1T, LDV1T, 0, 1, U1, LDU1, U2, LDU2, 0, 0,
-     $                   0, 0, 0, 0, 0, 0, WORK(1), -1, CHILDINFO )
+     $                   DUM1, V1T, LDV1T, DUM2, 1, U1, LDU1, U2,
+     $                   LDU2, DUM1, DUM1, DUM1, DUM1, DUM1,
+     $                   DUM1, DUM1, DUM1, WORK(1), -1, CHILDINFO
+     $                 )
             LBBCSD = INT( WORK(1) )
          ELSE IF( R .EQ. M-P ) THEN
-            CALL SORBDB3( M, P, Q, X11, LDX11, X21, LDX21, THETA, 0, 0,
-     $                    0, 0, WORK(1), -1, CHILDINFO )
+            CALL SORBDB3( M, P, Q, X11, LDX11, X21, LDX21, THETA,
+     $                    DUM1, DUM1, DUM1, DUM1, WORK(1), -1,
+     $                    CHILDINFO )
             LORBDB = INT( WORK(1) )
-            IF( P .GE. M-P-1 ) THEN
-               CALL SORGQR( P, P, Q, U1, LDU1, 0, WORK(1), -1,
+            IF( WANTU1 .AND. P .GT. 0 ) THEN
+               CALL SORGQR( P, P, Q, U1, LDU1, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, P )
-               LORGQROPT = INT( WORK(1) )
-            ELSE
-               CALL SORGQR( M-P-1, M-P-1, M-P-1, U2(2,2), LDU2, 0,
+               LORGQRMIN = MAX( LORGQRMIN, P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTU2 .AND. M-P .GT. 0 ) THEN
+               CALL SORGQR( M-P-1, M-P-1, M-P-1, U2(2,2), LDU2, DUM1,
      $                      WORK(1), -1, CHILDINFO )
-               LORGQRMIN = MAX( 1, M-P-1 )
-               LORGQROPT = INT( WORK(1) )
+               LORGQRMIN = MAX( LORGQRMIN, M-P-1 )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTV1T .AND. Q .GT. 0 ) THEN
+               CALL SORGLQ( Q, Q, R, V1T, LDV1T, DUM1, WORK(1), -1,
+     $                      CHILDINFO )
+               LORGLQMIN = MAX( LORGLQMIN, Q )
+               LORGLQOPT = MAX( LORGLQOPT, INT( WORK(1) ) )
             END IF
-            CALL SORGLQ( Q, Q, R, V1T, LDV1T, 0, WORK(1), -1,
-     $                   CHILDINFO )
-            LORGLQMIN = MAX( 1, Q )
-            LORGLQOPT = INT( WORK(1) )
             CALL SBBCSD( 'N', JOBV1T, JOBU2, JOBU1, 'T', M, M-Q, M-P,
-     $                   THETA, 0, 0, 1, V1T, LDV1T, U2, LDU2, U1, LDU1,
-     $                   0, 0, 0, 0, 0, 0, 0, 0, WORK(1), -1,
+     $                   THETA, DUM1, DUM2, 1, V1T, LDV1T, U2, LDU2,
+     $                   U1, LDU1, DUM1, DUM1, DUM1, DUM1,
+     $                   DUM1, DUM1, DUM1, DUM1, WORK(1), -1,
      $                   CHILDINFO )
             LBBCSD = INT( WORK(1) )
          ELSE
-            CALL SORBDB4( M, P, Q, X11, LDX11, X21, LDX21, THETA, 0, 0,
-     $                    0, 0, 0, WORK(1), -1, CHILDINFO )
+            CALL SORBDB4( M, P, Q, X11, LDX11, X21, LDX21, THETA,
+     $                    DUM1, DUM1, DUM1, DUM1, DUM1,
+     $                    WORK(1), -1, CHILDINFO )
             LORBDB = M + INT( WORK(1) )
-            IF( P .GE. M-P ) THEN
-               CALL SORGQR( P, P, M-Q, U1, LDU1, 0, WORK(1), -1,
+            IF( WANTU1 .AND. P .GT. 0 ) THEN
+               CALL SORGQR( P, P, M-Q, U1, LDU1, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, P )
-               LORGQROPT = INT( WORK(1) )
-            ELSE
-               CALL SORGQR( M-P, M-P, M-Q, U2, LDU2, 0, WORK(1), -1,
+               LORGQRMIN = MAX( LORGQRMIN, P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTU2 .AND. M-P .GT. 0 ) THEN
+               CALL SORGQR( M-P, M-P, M-Q, U2, LDU2, DUM1, WORK(1),
+     $                      -1, CHILDINFO )
+               LORGQRMIN = MAX( LORGQRMIN, M-P )
+               LORGQROPT = MAX( LORGQROPT, INT( WORK(1) ) )
+            END IF
+            IF( WANTV1T .AND. Q .GT. 0 ) THEN
+               CALL SORGLQ( Q, Q, Q, V1T, LDV1T, DUM1, WORK(1), -1,
      $                      CHILDINFO )
-               LORGQRMIN = MAX( 1, M-P )
-               LORGQROPT = INT( WORK(1) )
+               LORGLQMIN = MAX( LORGLQMIN, Q )
+               LORGLQOPT = MAX( LORGLQOPT, INT( WORK(1) ) )
             END IF
-            CALL SORGLQ( Q, Q, Q, V1T, LDV1T, 0, WORK(1), -1,
-     $                   CHILDINFO )
-            LORGLQMIN = MAX( 1, Q )
-            LORGLQOPT = INT( WORK(1) )
             CALL SBBCSD( JOBU2, JOBU1, 'N', JOBV1T, 'N', M, M-P, M-Q,
-     $                   THETA, 0, U2, LDU2, U1, LDU1, 0, 1, V1T, LDV1T,
-     $                   0, 0, 0, 0, 0, 0, 0, 0, WORK(1), -1,
+     $                   THETA, DUM1, U2, LDU2, U1, LDU1, DUM2, 1,
+     $                   V1T, LDV1T, DUM1, DUM1, DUM1, DUM1,
+     $                   DUM1, DUM1, DUM1, DUM1, WORK(1), -1,
      $                   CHILDINFO )
             LBBCSD = INT( WORK(1) )
          END IF
@@ -495,16 +525,16 @@
             CALL SORGLQ( Q-1, Q-1, Q-1, V1T(2,2), LDV1T, WORK(ITAUQ1),
      $                   WORK(IORGLQ), LORGLQ, CHILDINFO )
          END IF
-*   
+*
 *        Simultaneously diagonalize X11 and X21.
-*   
+*
          CALL SBBCSD( JOBU1, JOBU2, JOBV1T, 'N', 'N', M, P, Q, THETA,
-     $                WORK(IPHI), U1, LDU1, U2, LDU2, V1T, LDV1T, 0, 1,
-     $                WORK(IB11D), WORK(IB11E), WORK(IB12D),
+     $                WORK(IPHI), U1, LDU1, U2, LDU2, V1T, LDV1T,
+     $                DUM2, 1, WORK(IB11D), WORK(IB11E), WORK(IB12D),
      $                WORK(IB12E), WORK(IB21D), WORK(IB21E),
      $                WORK(IB22D), WORK(IB22E), WORK(IBBCSD), LBBCSD,
      $                CHILDINFO )
-*   
+*
 *        Permute rows and columns to place zero submatrices in
 *        preferred positions
 *
@@ -549,16 +579,16 @@
             CALL SORGLQ( Q, Q, R, V1T, LDV1T, WORK(ITAUQ1),
      $                   WORK(IORGLQ), LORGLQ, CHILDINFO )
          END IF
-*   
+*
 *        Simultaneously diagonalize X11 and X21.
-*   
+*
          CALL SBBCSD( JOBV1T, 'N', JOBU1, JOBU2, 'T', M, Q, P, THETA,
-     $                WORK(IPHI), V1T, LDV1T, 0, 1, U1, LDU1, U2, LDU2,
-     $                WORK(IB11D), WORK(IB11E), WORK(IB12D),
+     $                WORK(IPHI), V1T, LDV1T, DUM1, 1, U1, LDU1, U2,
+     $                LDU2, WORK(IB11D), WORK(IB11E), WORK(IB12D),
      $                WORK(IB12E), WORK(IB21D), WORK(IB21E),
      $                WORK(IB22D), WORK(IB22E), WORK(IBBCSD), LBBCSD,
      $                CHILDINFO )
-*   
+*
 *        Permute rows and columns to place identity submatrices in
 *        preferred positions
 *
@@ -604,16 +634,16 @@
             CALL SORGLQ( Q, Q, R, V1T, LDV1T, WORK(ITAUQ1),
      $                   WORK(IORGLQ), LORGLQ, CHILDINFO )
          END IF
-*   
+*
 *        Simultaneously diagonalize X11 and X21.
-*   
+*
          CALL SBBCSD( 'N', JOBV1T, JOBU2, JOBU1, 'T', M, M-Q, M-P,
-     $                THETA, WORK(IPHI), 0, 1, V1T, LDV1T, U2, LDU2, U1,
-     $                LDU1, WORK(IB11D), WORK(IB11E), WORK(IB12D),
-     $                WORK(IB12E), WORK(IB21D), WORK(IB21E),
-     $                WORK(IB22D), WORK(IB22E), WORK(IBBCSD), LBBCSD,
-     $                CHILDINFO )
-*   
+     $                THETA, WORK(IPHI), DUM1, 1, V1T, LDV1T, U2,
+     $                LDU2, U1, LDU1, WORK(IB11D), WORK(IB11E),
+     $                WORK(IB12D), WORK(IB12E), WORK(IB21D),
+     $                WORK(IB21E), WORK(IB22D), WORK(IB22E),
+     $                WORK(IBBCSD), LBBCSD, CHILDINFO )
+*
 *        Permute rows and columns to place identity submatrices in
 *        preferred positions
 *
@@ -673,16 +703,16 @@
             CALL SORGLQ( Q, Q, Q, V1T, LDV1T, WORK(ITAUQ1),
      $                   WORK(IORGLQ), LORGLQ, CHILDINFO )
          END IF
-*   
+*
 *        Simultaneously diagonalize X11 and X21.
-*   
+*
          CALL SBBCSD( JOBU2, JOBU1, 'N', JOBV1T, 'N', M, M-P, M-Q,
-     $                THETA, WORK(IPHI), U2, LDU2, U1, LDU1, 0, 1, V1T,
-     $                LDV1T, WORK(IB11D), WORK(IB11E), WORK(IB12D),
+     $                THETA, WORK(IPHI), U2, LDU2, U1, LDU1, DUM1, 1,
+     $                V1T, LDV1T, WORK(IB11D), WORK(IB11E), WORK(IB12D),
      $                WORK(IB12E), WORK(IB21D), WORK(IB21E),
      $                WORK(IB22D), WORK(IB22E), WORK(IBBCSD), LBBCSD,
      $                CHILDINFO )
-*   
+*
 *        Permute rows and columns to place identity submatrices in
 *        preferred positions
 *
